<?php
namespace Teknisa\CMP\Modules\Fornecimento\Modules\ApuracaoEfetivacao\Services;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class ApuracaoService {
    
    protected $connection;
    protected $newCode;
    
    public function __construct($entityManager, $newCode, $environment, $dataSourceManager) {
        $this->connection = $entityManager->getConnection();
        $this->newCode = $newCode;
        $this->environment = $environment;
        $this->dataSourceManager = $dataSourceManager;
    }
    
    public function beginTransaction() {
        $this->connection->beginTransaction();
    }
    
    public function commit() {
        $this->connection->commit();
    }
    
    public function rollback() {
        $this->connection->rollback();
    }
    
    public function generateCode($newCode) {
        $organizacao = $this->environment->getNrOrgTrab();
        $nrLanctoNfNota = $this->newCode->getCodeByTableName($newCode, $organizacao);
        return $nrLanctoNfNota;
    }
    
    public function qryAutGrupOp() {
        $param = array();
        $sql = "SELECT AUTGRUPOP.IDAUTGRUPOP, OPERGRUPOP.CDOPERADOR, OPERGRUPOP.NRORG FROM AUTGRUPOP, OPERGRUPOP WHERE AUTGRUPOP.IDAUTGRUPOP = 'PEME' AND AUTGRUPOP.CDGRUPOPER = OPERGRUPOP.CDGRUPOPER";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($param);
        return $stmt->fetchAll();
    }
    
    public function adqOptimizerMode() {
        $sql = "SELECT VALUE FROM V\$PARAMETER WHERE NAME = 'optimizer_mode'";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute();
        return $stmt->fetchAll()[0]["VALUE"];
    }
    
    public function adqNrlogcorteol($params) {
        $sql = "SELECT NRLOGCORTEOL, CDFILCENTCMP FROM APURPRECO WHERE (APURPRECO.CDFILCENTCMP = :P_CDCENTCMP) AND (APURPRECO.NRAPURPREC = :P_NRSEQAPUR) AND NRLOGCORTEOL IS NOT NULL";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
        return $stmt->fetchAll();
    }
    
    public function selectCotacaoWithParamNull($params) {
        $sql = "SELECT ITMAPURPREC.CDFORNITMAP, FORNECEDOR.NMRAZSOCFORN, FORNECEDOR.NMFANTFORN, FORNECEDOR.NRINSJURFORN, FORNECEDOR.IDTPIJURFORN, (ITMAPURPREC.CDREGITMAP ||' - ' ||REGIAO.NMREGIAO) AS CDNMREGIAO, (PRODUTO.CDARVPROD ||' - ' ||PRODUTO.NMPRODUTO) AS CDNMPRODUTO, ITMAPURPREC.NRSEQITMAP, ITMAPURPREC.CDCENTITMAP, ITMAPURPREC.CDREGITMAP, REGIAO.NMREGIAO, ITMAPURPREC.NRCOLEITMAP, PRODUTO.CDPRODESTO, ITMAPURPREC.CDPRODITMAP, PRODUTO.CDARVPROD, PRODUTO.NMPRODUTO, ITMAPURPREC.CDMARCITMAP, MARCA.NMMARCA, ITMAPURPREC.VRPRECUNIAP, ITMAPURPREC.VRPRECECOAP                                                                                              AS PRECOECON, ITMAPURPREC.VRPRECONPIAP                                                                                             AS PRECOECONPI, SUM(ITMAPURPREC.QTSOLACOMP)                                                                                          AS QTACOMP, MAX(ITMAPURPREC.IDITMSELAPUR)                                                                                        AS IDITMSELAPUR, DECODE (PARASUPRI.IDTIPOAPUR, NULL, ITMAPURPREC.VRPRECUNIAP, '1', ITMAPURPREC.VRPRECECOAP, ITMAPURPREC.VRPRECONPIAP) AS ORDENA, FORNECEDOR.SGUNIDADE, NVL(MINFORNREG.QTMINFORNREG, FORNECEDOR.QTMINAUTFOT) AS QTMINAUTFOT, DECODE(MINFORNREG.QTMINFORNREG, NULL, 'F', 'R')      AS IDQTMINFORN, NVL(MINFORNREG.VRMINFORNREG, FORNECEDOR.VRMINAUTFOR) AS VRMINAUTFOR, DECODE(MINFORNREG.VRMINFORNREG, NULL, 'F', 'R')      AS IDVRMINFORN, PRODUTO.VRFATOCONV, ITMAPURPREC.VRPRECUNIAP/PRODUTO.VRFATOCONV PRODUNDIVFATCO, 0 QTESTOSEGU, 0 CALC_ESTO, COLETAPR.DTINIVALCOL, COLETAPR.DTFINVALCOL, TO_CHAR(COLETAPR.DTINIVALCOL, 'DD/MM/YYYY') ||' - '|| TO_CHAR(COLETAPR.DTFINVALCOL, 'DD/MM/YYYY') AS PERIODO FROM ITMAPURPREC INNER JOIN PRODUTO ON ITMAPURPREC.CDPRODITMAP = PRODUTO.CDPRODUTO AND ITMAPURPREC.NRORG = PRODUTO.NRORG INNER JOIN COLETAPR ON COLETAPR.CDFILIAL = ITMAPURPREC.CDCENTITMAP AND COLETAPR.NRCOLEPRECO = ITMAPURPREC.NRCOLEITMAP AND COLETAPR.NRORG = ITMAPURPREC.NRORG INNER JOIN FORNECEDOR ON ITMAPURPREC.CDFORNITMAP = FORNECEDOR.CDFORNECED AND ITMAPURPREC.NRORG = FORNECEDOR.NRORG INNER JOIN REGIAO ON ITMAPURPREC.CDREGITMAP  = REGIAO.CDREGIAO AND ITMAPURPREC.NRORG  = REGIAO.NRORG INNER JOIN MARCA ON ITMAPURPREC.CDMARCITMAP = MARCA.CDMARCA AND ITMAPURPREC.NRORG = MARCA.NRORG INNER JOIN PARASUPRI ON ITMAPURPREC.CDCENTITMAP = PARASUPRI.CDFILIAL AND ITMAPURPREC.NRORG = PARASUPRI.NRORG LEFT JOIN MINFORNREG ON ITMAPURPREC.CDREGITMAP  = MINFORNREG.CDREGIAO AND ITMAPURPREC.CDFORNITMAP = MINFORNREG.CDFORNECED AND ITMAPURPREC.NRORG = MINFORNREG.NRORG WHERE ITMAPURPREC.CDCENTITMAP = :P_CDCENTCMP AND ITMAPURPREC.NRSEQITMAP  = :P_NRSEQAPUR AND ((PRODUTO.CDPRODESTO = :P_CDPRODESTO) OR (:P_CDPRODESTO = 'T')) GROUP BY ITMAPURPREC.CDFORNITMAP, FORNECEDOR.NMRAZSOCFORN, FORNECEDOR.NMFANTFORN, FORNECEDOR.NRINSJURFORN, FORNECEDOR.IDTPIJURFORN, ITMAPURPREC.NRSEQITMAP, ITMAPURPREC.CDCENTITMAP, ITMAPURPREC.CDREGITMAP, REGIAO.NMREGIAO, ITMAPURPREC.NRCOLEITMAP, PRODUTO.CDPRODESTO, ITMAPURPREC.CDPRODITMAP, PRODUTO.CDARVPROD, PRODUTO.NMPRODUTO, ITMAPURPREC.CDMARCITMAP, MARCA.NMMARCA, ITMAPURPREC.VRPRECUNIAP, ITMAPURPREC.VRPRECECOAP, ITMAPURPREC.VRPRECONPIAP, DECODE (PARASUPRI.IDTIPOAPUR, NULL, ITMAPURPREC.VRPRECUNIAP, '1', ITMAPURPREC.VRPRECECOAP, ITMAPURPREC.VRPRECONPIAP), FORNECEDOR.SGUNIDADE, NVL(MINFORNREG.QTMINFORNREG, FORNECEDOR.QTMINAUTFOT), DECODE(MINFORNREG.QTMINFORNREG, NULL, 'F', 'R')     , NVL(MINFORNREG.VRMINFORNREG, FORNECEDOR.VRMINAUTFOR), DECODE(MINFORNREG.VRMINFORNREG, NULL, 'F', 'R')     , PRODUTO.VRFATOCONV, COLETAPR.DTINIVALCOL, COLETAPR.DTFINVALCOL ORDER BY DECODE (PARASUPRI.IDTIPOAPUR, NULL, ITMAPURPREC.VRPRECUNIAP, '1', ITMAPURPREC.VRPRECECOAP, ITMAPURPREC.VRPRECONPIAP)";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
        return $stmt->fetchAll();
    }
    
    public function selectCotacaoWithParamNotNull($params) {
        $sql = "SELECT ITMAPURPREC.CDFORNITMAP, ITMAPURPREC.NMRAZSOCFORN, ITMAPURPREC.NMFANTFORN, ITMAPURPREC.NRINSJURFORN, ITMAPURPREC.IDTPIJURFORN, ITMAPURPREC.CDNMREGIAO, ITMAPURPREC.CDNMPRODUTO, ITMAPURPREC.NRSEQITMAP, ITMAPURPREC.CDCENTITMAP, ITMAPURPREC.CDREGITMAP, ITMAPURPREC.NMREGIAO, ITMAPURPREC.NRCOLEITMAP, ITMAPURPREC.CDPRODITMAP, ITMAPURPREC.CDPRODESTO, ITMAPURPREC.CDARVPROD, ITMAPURPREC.NMPRODUTO, ITMAPURPREC.CDMARCITMAP, ITMAPURPREC.NMMARCA, ITMAPURPREC.VRPRECUNIAP, ITMAPURPREC.PRECOECON, ITMAPURPREC.PRECOECONPI, SUM(ITMAPURPREC.QTSOLACOMP)   AS QTACOMP, MAX(ITMAPURPREC.IDITMSELAPUR) AS IDITMSELAPUR, ITMAPURPREC.ORDENA, ITMAPURPREC.SGUNIDADE, ITMAPURPREC.QTMINAUTFOT, ITMAPURPREC.IDQTMINFORN, ITMAPURPREC.VRMINAUTFOR, ITMAPURPREC.IDVRMINFORN, ITMAPURPREC.VRFATOCONV, ITMAPURPREC.PRODUNDIVFATCO, ITMAPURPREC.QTESTOSEGU, ITMAPURPREC.CALC_ESTO, ITMAPURPREC.DTINIVALCOL, ITMAPURPREC.DTFINVALCOL, TO_CHAR(ITMAPURPREC.DTINIVALCOL, 'DD/MM/YYYY') ||' - '|| TO_CHAR(ITMAPURPREC.DTFINVALCOL, 'DD/MM/YYYY') AS PERIODO FROM (SELECT ITMAPURPREC.CDFORNITMAP, FORNECEDOR.NMRAZSOCFORN, FORNECEDOR.NMFANTFORN, FORNECEDOR.NRINSJURFORN, FORNECEDOR.IDTPIJURFORN, (ITMAPURPREC.CDREGITMAP ||' - ' ||REGIAO.NMREGIAO) AS CDNMREGIAO, (PRODUTO.CDARVPROD ||' - ' ||PRODUTO.NMPRODUTO) AS CDNMPRODUTO, ITMAPURPREC.NRSEQITMAP, ITMAPURPREC.CDCENTITMAP, ITMAPURPREC.CDREGITMAP, REGIAO.NMREGIAO, ITMAPURPREC.NRCOLEITMAP, ITMAPURPREC.CDPRODITMAP, PRODUTO.CDPRODESTO, PRODUTO.CDARVPROD, PRODUTO.NMPRODUTO, ITMAPURPREC.CDMARCITMAP, MARCA.NMMARCA, ITMAPURPREC.VRPRECUNIAP, ITMAPURPREC.VRPRECECOAP  AS PRECOECON, ITMAPURPREC.VRPRECONPIAP AS PRECOECONPI, ITMAPURPREC.QTSOLACOMP, ITMAPURPREC.IDITMSELAPUR, DECODE (PARASUPRI.IDTIPOAPUR, NULL, ITMAPURPREC.VRPRECUNIAP, '1', ITMAPURPREC.VRPRECECOAP, ITMAPURPREC.VRPRECONPIAP) AS ORDENA, FORNECEDOR.SGUNIDADE, NVL(MINFORNREG.QTMINFORNREG, FORNECEDOR.QTMINAUTFOT) AS QTMINAUTFOT, DECODE(MINFORNREG.QTMINFORNREG, NULL, 'F', 'R')      AS IDQTMINFORN, NVL(MINFORNREG.VRMINFORNREG, FORNECEDOR.VRMINAUTFOR) AS VRMINAUTFOR, DECODE(MINFORNREG.VRMINFORNREG, NULL, 'F', 'R')      AS IDVRMINFORN, PRODUTO.VRFATOCONV, ITMAPURPREC.VRPRECUNIAP/PRODUTO.VRFATOCONV PRODUNDIVFATCO, CASE WHEN CEIL(NVL(ESTOSEGU.QTESTOSEGU, 0)/PRODUTO.VRFATOCONV) < 0 THEN 0 ELSE CEIL(NVL(ESTOSEGU.QTESTOSEGU, 0)/PRODUTO.VRFATOCONV) END AS QTESTOSEGU, CASE WHEN CEIL(NVL(ESTOSEGU.QTESTOSEGU - CALC_ESTOQ_CONS(:P_CDFILIOPELOG, RELACPRODOL.CDPRODARVOL, SYSDATE + 1), 0)/PRODUTO.VRFATOCONV) < 0 THEN 0 ELSE CEIL(NVL(ESTOSEGU.QTESTOSEGU - CALC_ESTOQ_CONS(:P_CDFILIOPELOG, RELACPRODOL.CDPRODARVOL, SYSDATE + 1), 0)/PRODUTO.VRFATOCONV) END AS CALC_ESTO, COLETAPR.DTINIVALCOL, COLETAPR.DTFINVALCOL FROM ITMAPURPREC INNER JOIN PRODUTO ON ITMAPURPREC.CDPRODITMAP = PRODUTO.CDPRODUTO AND ITMAPURPREC.NRORG = PRODUTO.NRORG INNER JOIN COLETAPR ON COLETAPR.CDFILIAL = ITMAPURPREC.CDCENTITMAP AND COLETAPR.NRCOLEPRECO = ITMAPURPREC.NRCOLEITMAP AND COLETAPR.NRORG = ITMAPURPREC.NRORG INNER JOIN FORNECEDOR ON ITMAPURPREC.CDFORNITMAP = FORNECEDOR.CDFORNECED AND ITMAPURPREC.NRORG = FORNECEDOR.NRORG INNER JOIN REGIAO ON ITMAPURPREC.CDREGITMAP  = REGIAO.CDREGIAO AND ITMAPURPREC.NRORG  = REGIAO.NRORG INNER JOIN MARCA ON ITMAPURPREC.CDMARCITMAP = MARCA.CDMARCA AND ITMAPURPREC.NRORG = MARCA.NRORG INNER JOIN PARASUPRI ON ITMAPURPREC.CDCENTITMAP = PARASUPRI.CDFILIAL AND ITMAPURPREC.NRORG = PARASUPRI.NRORG LEFT JOIN MINFORNREG ON ITMAPURPREC.CDREGITMAP = MINFORNREG.CDREGIAO AND ITMAPURPREC.CDFORNITMAP = MINFORNREG.CDFORNECED AND ITMAPURPREC.NRORG = MINFORNREG.NRORG LEFT JOIN RELACPRODOL ON ITMAPURPREC.CDPRODITMAP = RELACPRODOL.CDPRARVORIG AND ITMAPURPREC.CDMARCITMAP = RELACPRODOL.CDMARCA AND ITMAPURPREC.NRORG = RELACPRODOL.NRORG LEFT JOIN (SELECT QTESTOSEGU, CDPRODESTO, NRORG FROM ESTOSEGU E WHERE E.CDFILIAL = :P_CDFILIOPELOG) ESTOSEGU ON PRODUTO.CDPRODESTO = ESTOSEGU.CDPRODESTO AND PRODUTO.NRORG = ESTOSEGU.NRORG WHERE ITMAPURPREC.CDCENTITMAP = :P_CDCENTCMP AND ITMAPURPREC.NRSEQITMAP = :P_NRSEQAPUR AND ((PRODUTO.CDPRODESTO = :P_CDPRODESTO) OR (:P_CDPRODESTO = 'T')) ) ITMAPURPREC GROUP BY ITMAPURPREC.CDFORNITMAP, ITMAPURPREC.NMRAZSOCFORN, ITMAPURPREC.NMFANTFORN, ITMAPURPREC.NRINSJURFORN, ITMAPURPREC.IDTPIJURFORN, ITMAPURPREC.CDNMREGIAO, ITMAPURPREC.CDNMPRODUTO, ITMAPURPREC.NRSEQITMAP, ITMAPURPREC.CDCENTITMAP, ITMAPURPREC.CDREGITMAP, ITMAPURPREC.NMREGIAO, ITMAPURPREC.NRCOLEITMAP, ITMAPURPREC.CDPRODITMAP, ITMAPURPREC.CDPRODESTO, ITMAPURPREC.CDARVPROD, ITMAPURPREC.NMPRODUTO, ITMAPURPREC.CDMARCITMAP, ITMAPURPREC.NMMARCA, ITMAPURPREC.VRPRECUNIAP, ITMAPURPREC.PRECOECON, ITMAPURPREC.PRECOECONPI, ITMAPURPREC.ORDENA, ITMAPURPREC.SGUNIDADE, ITMAPURPREC.QTMINAUTFOT, ITMAPURPREC.IDQTMINFORN, ITMAPURPREC.VRMINAUTFOR, ITMAPURPREC.IDVRMINFORN, ITMAPURPREC.VRFATOCONV, ITMAPURPREC.PRODUNDIVFATCO, ITMAPURPREC.QTESTOSEGU, ITMAPURPREC.CALC_ESTO, ITMAPURPREC.DTINIVALCOL, ITMAPURPREC.DTFINVALCOL ORDER BY QTACOMP, ORDENA";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
        return $stmt->fetchAll();
    }
    
    public function updateMelhorCotacao($param) {
        $sql = "UPDATE ITMAPURPREC SET IDITMSELAPUR = :IDITMSELAPUR, TXOBESCMANAP = TXOBESCMANAP||:TXOBESCMANAP WHERE CDCENTITMAP = :OLD_CDCENTITMAP AND CDPRODITMAP = :OLD_CDPRODITMAP AND NRCOLEITMAP = :OLD_NRCOLEITMAP AND NRSEQITMAP = :OLD_NRSEQITMAP AND CDFORNITMAP = :OLD_CDFORNITMAP AND CDMARCITMAP = :OLD_CDMARCITMAP AND CDREGITMAP = :OLD_CDREGITMAP";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($param);
    }
    
    public function qryOperRelacAux($params) {
        $sql = "SELECT CDOPERADOR, IDEXSENHCMPOPER, IDCOTCMPOPER, IDAPURCMPOPER, IDAFCMPOPER FROM COMPROPER WHERE CDCOMPRADOR = :P_CDCOMPRADOR";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
        return $stmt->fetchAll();
    }
    
    public function centralCompras($param) {
        $sql = "SELECT F.CDFILIAL, F.NMFILIAL, F2.CDFILIAL AS CDFILCENTCMP, F2.NMFILIAL AS NMFILCENTCMP, F2.NRORG FROM FILIAL F, FILIAL F2 WHERE F.CDFILIAL = :CDFILIAL AND F2.CDFILIAL = NVL(F.CDFILCENTCMP, F.CDFILIAL)";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($param);
        return $stmt->fetchAll();
    }
    
    public function adqParam($param) {
        $sql = "SELECT PARASUPRI.IDEXJUSTESCM, PARAMGERAL.IDSISDOS, PARAMGERAL.NRORG, PARASUPRI.IDAPUREFESOLEXT, TRUNC(SYSDATE) AS DATA, PARASUPRI.NRDIAANTENTRAF FROM PARAMGERAL, PARASUPRI WHERE PARASUPRI.CDFILIAL = :CDFILCENTCMP";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($param);
        return $stmt->fetchAll();
    }
    
    public function adqParamCmp($param) {
        $sql = "SELECT CDFILIAL , IDJUSSOLEXT , IDBLOQALTNEC, IDAUDALTSOL, IDULTIDTULTI, IDULTCOMPSOL, IDULTDESTSOL, CDGRUPOPER , IDULTIDTVALQ, IDVALDTAF , IDBLOQUEAAF , IDMARSEMQUA, IDTIPOAPUR, IDPROSEMMAR, IDPROSEMFOR , IDUTQTDEVOLU, IDBAIXSOLDFS, IDCONSPRSOL, IDEXJUSTESCM , IDBLOQALTSOLAUT, NRORG FROM PARASUPRI WHERE INSTR(:CDFILCENTCMP, CDFILIAL) > 0";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($param);
        return $stmt->fetchAll();
    }
    
    public function adqOperadorComprador($param) {
        $sql = "SELECT
                        CDOPERADOR,
                        IDEXSENHCMPOPER,
                        IDCOTCMPOPER,
                        IDAPURCMPOPER,
                        IDAFCMPOPER
                    FROM
                        COMPROPER
                    WHERE
                            CDCOMPRADOR = :P_CDCOMPRADOR
                        AND CDOPERADOR = :P_CDOPERADOR
                    UNION
                    SELECT
                        CDOPERADOR,
                        'N'  IDEXSENHCMPOPER,
                        'S'  IDCOTCMPOPER,
                        'S'  IDAPURCMPOPER,
                        'S'  IDAFCMPOPER
                    FROM
                        COMPRADOR
                    WHERE
                        CDOPERADOR = :P_CDOPERADOR";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($param);
        return $stmt->fetchAll();
    }
    
    public function adqEfetivaAF($params) {
        $sql = "SELECT NRSEQAPUR FROM EFETIVAAF WHERE CDOPERADOR = :P_CDCOMPRADOR AND DTAUTOFORN = TRUNC(SYSDATE) AND CDCENTCMP = :P_CDCENTCMP AND NRSEQAPUR = :P_NRSEQAPUR AND CDFILLANC = :P_CDFILLANC AND CDOPERALANC = :P_CDOPERALANC";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($param);
        return $stmt->fetchAll();
    }
    
    public function executeAPURA_EFETIVACAO($params) {
        $procedure = new StoredProcedure($this->connection, 'APURA_EFETIVACAO');
        $procedure->addParam(new Param('P_CDCENTCMP',      Param::PARAM_INPUT, $params['P_CDCENTCMP'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRSEQAPUR',      Param::PARAM_INPUT, $params['P_NRSEQAPUR'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_PRODINI',        Param::PARAM_INPUT, $params['P_PRODINI'],   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_PRODFIN',        Param::PARAM_INPUT, $params['P_PRODFIN'],   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTINI',          Param::PARAM_INPUT, $params['P_DTINI'],     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTFIN',          Param::PARAM_INPUT, $params['P_DTFIN'],     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR',     Param::PARAM_INPUT, $params['P_CDOPERADOR'],Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDREGIAO',       Param::PARAM_INPUT, $params['P_CDREGIAO'],  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIAL',       Param::PARAM_INPUT, $params['P_CDFILIAL'],  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTPROCESSA',     Param::PARAM_INPUT, $params['P_DTPROCESSA'],Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_FILIPROC',       Param::PARAM_INPUT, $params['P_FILIPROC'],  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDGRUPFILI',     Param::PARAM_INPUT, $params['P_CDGRUPFILI'],Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_OPTIMIZER_MODE', Param::PARAM_INPUT, "CHOOSE",               Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_ROWSAFFECT',     Param::PARAM_OUTPUT));
        $procedure->addParam(new Param('P_CDOPERLANC',     Param::PARAM_INPUT, $params['P_CDOPERLANC'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCOMPRADOR',    Param::PARAM_INPUT, $params['P_CDCOMPRADOR'],Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_LISTAFILIAL',    Param::PARAM_INPUT, $params['P_LISTAFILIAL'],Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_EXTRA',          Param::PARAM_INPUT, $params['P_EXTRA'],      Param::PARAM_TYPE_STR));
        $response = $procedure->execute();
        return $response;
    }
    
    public function executeAPURA_MELHOR_COTACAO($params) {
        $procedure = new StoredProcedure($this->connection, 'APURA_MELHOR_COTACAO');
        $procedure->addParam(new Param('P_CDCENTCMP', Param::PARAM_INPUT, $params['P_CDCENTCMP'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRSEQAPUR', Param::PARAM_INPUT, $params['P_NRSEQAPUR'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_SISTEMA', Param::PARAM_INPUT, $params['P_SISTEMA'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_TIPO', Param::PARAM_INPUT, $params['P_TIPO'], Param::PARAM_TYPE_STR));
        $response = $procedure->execute();
        return $response;
    }
    
    public function adqItemCole($params) {
        $sql = "SELECT MAX(CASE PARASUPRI.IDUTQTDISPGERAL WHEN 'N' THEN '1' ELSE CASE FORNECEDOR.IDUTQTDISPFORN WHEN 'N' THEN '1' ELSE CASE WHEN NVL(PRMARCPEDCOL.QTDISPPRODMARC, 0) > 0 THEN '0' ELSE '1' END END END) QTDISPPRODMARC, MAX(CASE NVL(COLETAPR.IDSITUCOLETA, 'C') WHEN 'C' THEN '0' ELSE '1' END) IDSITUCOLETA, MAX(CASE NVL(FORNECEDOR.IDTIPOFORN, 'N') WHEN 'N' THEN '0' ELSE '1' END) IDTIPOFORN, MAX(CASE NVL(FORNECEDOR.IDSITUCADA, 'A') WHEN 'A' THEN '0' ELSE '1' END) IDSITUCADA FROM COLETAPR, PRODUTO, FORNECEDOR, ITEMCOLE, PRMARCPEDCOL, PARASUPRI WHERE COLETAPR.CDFILIAL = :P_CDCENTCMP AND PARASUPRI.CDFILIAL = :P_CDCENTCMP AND (COLETAPR.CDOPERADOR = :P_CDOPERADOR OR :P_CDOPERADOR = 'T') AND PRODUTO.CDARVPROD BETWEEN :P_CDPRODINI AND :P_CDPRODFIN AND COLETAPR.CDFILIAL = ITEMCOLE.CDFILIAL AND COLETAPR.NRCOLEPRECO = ITEMCOLE.NRCOLEPRECO AND PRODUTO.CDPRODUTO = ITEMCOLE.CDPRODUTO AND PRMARCPEDCOL. CDFILIAL(+) = ITEMCOLE.CDFILIAL AND PRMARCPEDCOL.NRPEDIDOCOL(+) = ITEMCOLE.NRPEDIDOCOL AND PRMARCPEDCOL.CDPRODUTO(+) = ITEMCOLE.CDPRODUTO AND PRMARCPEDCOL.CDMARCA(+) = ITEMCOLE.CDMARCA AND (ITEMCOLE.IDITMLISPREC = :P_ITEM OR :P_ITEM = 'T') AND ITEMCOLE.VRPRECUNIITM <> 0 AND (INSTR(:P_CDREGIAO, ITEMCOLE.CDREGIAO) <> 0 OR :P_CDREGIAO = 'T') AND ITEMCOLE.DTINIVALITM >= :P_DATAINI AND ITEMCOLE.DTFINVALITM <= :P_DATAFIN AND FORNECEDOR.CDFORNECED = ITEMCOLE.CDFORNECED";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
        
        return $stmt->fetchAll()[0];
    }
    
    public function acmAtuaPrec($params) {
        $sql = "UPDATE ITEMCOLE SET VRPRECUNIITM = :P_VRPREC WHERE ITEMCOLE.CDFILIAL = :P_CDCENTCMP AND ITEMCOLE.CDPRODUTO = :P_CDPRODUTO AND ITEMCOLE.CDMARCA = :P_CDMARCA AND ITEMCOLE.CDFORNECED = :P_CDFORNECED AND ITEMCOLE.CDREGIAO = :P_CDREGIAO AND ITEMCOLE.NRCOLEPRECO = :P_NRCOLEPRECO";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
    }
    
    public function adqPrecEco($params) {
        $sql = "SELECT VRPRECECOITM, VRPRECOITMPI, VRPRECUNIITM FROM ITEMCOLE WHERE ITEMCOLE.CDFILIAL = :P_CDCENTCMP AND ITEMCOLE.CDPRODUTO = :P_CDPRODUTO AND ITEMCOLE.CDMARCA = :P_CDMARCA AND ITEMCOLE.CDFORNECED = :P_CDFORNECED AND ITEMCOLE.CDREGIAO = :P_CDREGIAO AND ITEMCOLE.NRCOLEPRECO = :P_NRCOLEPRECO";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
        
        return $stmt->fetchAll()[0];
    }
    
    public function acmAtuaPrecEco($params) {
        $sql = "UPDATE ITMAPURPREC SET VRPRECECOAP = :P_VRPREC, VRPRECONPIAP = :P_VRPRECPI, VRPRECUNIAP = :P_VRPRE WHERE NRSEQITMAP = :P_NRSEQITMAP AND CDCENTITMAP = :P_CDCENTCMP AND CDREGITMAP = :P_CDREGIAO AND CDFORNITMAP = :P_CDFORNECED AND NRCOLEITMAP = :P_NRCOLEPRECO AND CDPRODITMAP = :P_CDPRODUTO AND CDMARCITMAP = :P_CDMARCA";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
    }
    
    public function executeIMPOSTO_ITEMCOL($params) {
        $procedure = new StoredProcedure($this->connection, 'IMPOSTO_ITEMCOL');
        $procedure->addParam(new Param('P_EMPRESA', Param::PARAM_INPUT, $params['P_EMPRESA'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_INSCESTA', Param::PARAM_INPUT, $params['P_INSCESTA'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_FORNECEDOR', Param::PARAM_INPUT, $params['P_FORNECEDOR'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_FILCENT', Param::PARAM_INPUT, $params['P_FILCENT'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_LANCCOLP', Param::PARAM_INPUT, $params['P_LANCCOLP'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFORNECED', Param::PARAM_INPUT, $params['P_CDFORNECED'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDREGIAO', Param::PARAM_INPUT, $params['P_CDREGIAO'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDPRODUTO', Param::PARAM_INPUT, $params['P_CDPRODUTO'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDMARCA', Param::PARAM_INPUT, $params['P_CDMARCA'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_INIVALID', Param::PARAM_INPUT, $params['P_INIVALID'], Param::PARAM_TYPE_STR));
        $response = $procedure->execute();
        return $response;
    }
    
    public function insertOnMOVOPITEMCOT($params) {
        $sql = "INSERT INTO MOVOPITEMCOT(CDFILIALMCOT, NRCOLEPRECOMCOT, CDMARCAMCOT, CDREGIAOMCOT, CDFORNECEDMCOT, CDPRODUTOMCOT, DTOPERAITMCOT, CDOPERADORMCOT, DSALTERITMCOT, DSJUSTITMCOT, DSRDMUSUMCOT, DSENDUSUMCOT, DSNMUSUMCOT, DSMETUSUMCOT, NRORG, IDATIVO) VALUES ( :P_CDFILIALMCOT, :P_NRCOLEPRECOMCOT, :P_CDMARCAMCOT, :P_CDREGIAOMCOT, :P_CDFORNECEDMCOT, :P_CDPRODUTOMCOT, TO_DATE(:P_DTOPERAITMCOT, 'DD/MM/YYYY HH24:MI:SS'), :P_CDOPERADORMCOT, :P_DSALTERITMCOT, :P_DSJUSTITMCOT, :P_DSRDMUSUMCOT, :P_DSENDUSUMCOT, :P_DSNMUSUMCOT, :P_DSMETUSUMCOT, :P_NRORG, :P_IDATIVO)";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
    }
    
    public function adqEmitenteAF($params) {
        $sql = "SELECT EMITENTEAF.CDEMPRESA FROM EMITENTEAF, PRODUTO, PRODUTO P WHERE P.CDPRODUTO = :P_CDPRODUTO AND PRODUTO.CDPRODUTO = EMITENTEAF.CDPRODUTO AND EMITENTEAF.CDFILIAL = :P_CDFILIAL AND P.CDARVPROD LIKE PRODUTO.CDARVPROD || '%'";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
        
        return $stmt->fetchAll();
    }
    
    public function adqEmpresa($params) {
        $sql = "SELECT EMPRESA.CDEMPRESA, MIN(INSCRMUNICIP.CDINSCMUNI) AS CDINSCMUNI, MIN(INSCRESTAD.CDINSCESTA) AS CDINSCESTA FROM EMPRESA, INSCRMUNICIP, INSCRESTAD WHERE EMPRESA.CDEMPRESA = :P_CDEMPRESA AND EMPRESA.CDEMPRESA = INSCRMUNICIP.CDEMPRESA AND EMPRESA.CDEMPRESA = INSCRESTAD.CDEMPRESA GROUP BY EMPRESA.CDEMPRESA, EMPRESA.NRINSCESTA";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
        
        return $stmt->fetchAll();
    }
    
    public function acmUpdItmApurPrec($params) {
        $sql = "update ITMAPURPREC set QTACOMP = :QTACOMP, IDITMSELAPUR = :IDITMSELAPUR, TXOBESCMANAP = :TXOBESCMANAP where CDCENTITMAP = :OLD_CDCENTITMAP and DTENTRAPUT = :OLD_DTENTRAPUT and CDPRODITMAP = :OLD_CDPRODITMAP and NRCOLEITMAP = :OLD_NRCOLEITMAP and NRSEQITMAP = :OLD_NRSEQITMAP and CDFILIAP = :OLD_CDFILIAP and CDFORNITMAP = :OLD_CDFORNITMAP and CDMARCITMAP = :OLD_CDMARCITMAP and CDREGITMAP = :OLD_CDREGITMAP";
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
    }
}
