<?php
namespace Teknisa\CMP\Services;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Setforms2\Shared\ORM\DateTime;
use Zeedhi\Framework\DTO\Response\Notification;

class NecessidadeComprasService
{
    protected $crudService;
    protected $entityManager;
    protected $newCode;
    protected $environment;
    protected $dataSourceManager;
    
    const PARASUPRI         = 'parasupri';
    const PARCMPSUP         = 'parcmpsup';
    const PARAMGERAL        = 'paramgeral';
    const CDFILCENTCMP      = 'central_compras';
    const FILTROCOMPSIMPL   = 'compra_simplificada';
    
    public function __construct($crudService, $entityManager, $newCode, $environment, $dataSourceManager)
    {
        $this->crudService = $crudService;
        $this->connection = $entityManager->getConnection();
        $this->newCode = $newCode;
        $this->environment = $environment;
        $this->dataSourceManager = $dataSourceManager;
    }
    
    public function beginTransaction()
    {
        $this->connection->beginTransaction();
    }
    
    public function commit()
    {
        $this->connection->commit();
    }
    
    public function rollback()
    {
        $this->connection->rollback();
    }
    
    public function getCentralCompras()
	{
	    $conditions = array(
            ["columnName" => "CDFILIAL", "operator" => "=", "value" => $this->environment->getCdFilial()],
            ["columnName" => "NRORG",    "operator" => "=", "value" => $this->environment->getNrOrg()   ]
        );
        $central = $this->crudService->getFilteredDataSet(self::CDFILCENTCMP, $conditions);
        return $central;
	}
    
    public function getParaSupri($cdfilial)
	{
		$conditions = array(
            ["columnName" => "CDFILIAL", "operator" => "=", "value" => $this->environment->getCdFilial()],
            ["columnName" => "NRORG",    "operator" => "=", "value" => $this->environment->getNrOrg()   ]
        );
        $parasupri = $this->crudService->getFilteredDataSet(self::PARASUPRI, $conditions);
        return $parasupri;
	}

	public function getParCmpSup()
	{
		$conditions = array(
            ["columnName" => "CDFILIAL", "operator" => "=", "value" => $this->environment->getCdFilial()],
            ["columnName" => "NRORG",    "operator" => "=", "value" => $this->environment->getNrOrg()   ]
        );
        $parcmpsup = $this->crudService->getFilteredDataSet(self::PARCMPSUP, $conditions);
        return $parcmpsup;
	}
	
	public function getParamGeral()
	{
		$conditions = array(
            ["columnName" => "NRORG", "operator" => "=", "value" => $this->environment->getNrOrg()]
        );
        $paramgeral = $this->crudService->getFilteredDataSet(self::PARAMGERAL, $conditions);
        return $paramgeral;
	}
    
    private function getLimiSegu($cdfilial, $cdprodini, $cdprodfin, $groupfili)
    {
        $params = array(
    		"P_CDGRUPFILI" => $groupfili,
    		"P_CDFILIAL" => $cdfilial,
    		"P_CDPRODUTOINI" => $cdprodini,
    		"P_CDPRODUTOFIN" => $cdprodfin,
    		"NRORG" => $this->environment->getNrorg()
    	);

    	$sql = "SELECT MAX(NRLIMISEGU) AS MAXLIMISEG
    	          FROM ( SELECT CDFILIAL, NRORG
    	                   FROM FILGRUPFILI
    	                  WHERE FILGRUPFILI.CDGRUPFILI = :P_CDGRUPFILI
    	                    AND NRORG = :NRORG
    	                    AND (INSTR(:P_CDFILIAL, FILGRUPFILI.CDFILIAL) > 0 OR :P_CDFILIAL = 'T')
    	                  UNION
    	                 SELECT CDFILIAL, NRORG
    	                   FROM FILIAL
    	                  WHERE :P_CDGRUPFILI IS NULL
    	                    AND NRORG = :NRORG
    	                    AND (INSTR(:P_CDFILIAL, FILIAL.CDFILIAL) > 0 OR :P_CDFILIAL = 'T')) FILGRUPFILI, 
    	               LIMISEGPROD, 
    	               ( SELECT CDPRODUTO, NRORG
    	                   FROM PRODUTO
    	                  WHERE PRODUTO.CDARVPROD BETWEEN :P_CDPRODUTOINI AND RPAD(:P_CDPRODUTOFIN, 20, 'Z')
    	                    AND NRORG = :NRORG
    	                  UNION
    	                 SELECT CDPRODUTO, NRORG
    	                   FROM PRODUTO
    	                  WHERE :P_CDPRODUTOINI LIKE PRODUTO.CDARVPROD ||'%'
    	                    AND NRORG = :NRORG) PRODUTO
    	         WHERE LIMISEGPROD.CDPRODUTO = PRODUTO.CDPRODUTO
    	           AND LIMISEGPROD.NRORG = PRODUTO.NRORG
    	           AND FILGRUPFILI.CDFILIAL = LIMISEGPROD.CDFILIAL
    	           AND FILGRUPFILI.NRORG = LIMISEGPROD.NRORG";
    	$query = $this->connection->prepare($sql);
    	$query->execute($params);
    	$limi = $query->fetchAll();
    	
    	if (!empty($limi[0])) {
    	    return $limi[0];
    	}
    }
    
    private function getLimiSeguSimplificado($nrfiltro)
    {
        $params = array(
    		"P_NRFILTROCOMPSFDA" => $nrfiltro,
    		"NRORG" => $this->environment->getNrorg()
    	);

    	$sql = "SELECT MAX(NRLIMISEGU) AS MAXLIMISEG
                  FROM LIMISEGPROD, 
                       (
                            SELECT CDPRODUTO, NRORG
                              FROM PRODCOMPSIMPL
                             WHERE NRFILTROCOMPSFDA = :P_NRFILTROCOMPSFDA
                             UNION
                            SELECT PRODUTO.CDPRODUTO, PRODUTO.NRORG
                              FROM PRODCOMPSIMPL JOIN PRODUTO ON PRODCOMPSIMPL.CDARVPROD LIKE PRODUTO.CDARVPROD ||'%'
                                                             AND PRODCOMPSIMPL.NRORG LIKE PRODUTO.NRORG
                             WHERE PRODCOMPSIMPL.NRFILTROCOMPSFDA = :P_NRFILTROCOMPSFDA
                       ) PRODUTO,
                       (
                            SELECT CDFILIAL, NRORG
                              FROM FILICOMPSIMPL
                             WHERE NRFILTROCOMPSFDA = :P_NRFILTROCOMPSFDA
                       ) FILGRUPFILI
                 WHERE LIMISEGPROD.CDPRODUTO = PRODUTO.CDPRODUTO
                   AND LIMISEGPROD.NRORG     = PRODUTO.NRORG
                   AND FILGRUPFILI.CDFILIAL  = LIMISEGPROD.CDFILIAL
                   AND FILGRUPFILI.NRORG     = LIMISEGPROD.NRORG
                   AND LIMISEGPROD.NRORG     = :NRORG";
    	$query = $this->connection->prepare($sql);
    	$query->execute($params);
    	$limi = $query->fetchAll();
    	
    	if (!empty($limi[0])) {
    	    return $limi[0];
    	}
    }
    
    /**
     * Soma meses a data passada como parâmetro
     * 
     * @param {DateTime} $date Data que será somada
     * @param {int} $monthToAdd Quantidade de meses a adicionar
     * 
     * @return {DateTime} Data passada por parâmetro, somada à quantidade de meses
     * 
    */
    private function somaMes($date, $monthToAdd)
    {
        $year = $date->format('Y');
        $month = $date->format('m');
        $day = $date->format('d');
        
        $year += floor($monthToAdd/12);
        $monthToAdd = $monthToAdd%12;
        $month += $monthToAdd;
        
        if($month > 12) {
            $year ++;
            $month = $month % 12;
            if($month === 0)
                $month = 12;
        }
        
        if(!checkdate($month, $day, $year)) {
            $newDate = DateTime::createFromFormat('Y-n-j', $year.'-'.$month.'-1');
            $newDate->modify('last day of');
        }else {
            $newDate = DateTime::createFromFormat('Y-n-d', $year.'-'.$month.'-'.$day);
        }   
        
        return $newDate;
    }
    
    
    private function getCompraSimplificada($nrfiltrocompra)
    {
        $conditions = array(
            ["columnName" => "NRFILTROCOMPSFDA", "operator" => "=", "value" => $nrfiltrocompra],
            ["columnName" => "NRORG",            "operator" => "=", "value" => $this->environment->getNrOrg()]
        );
        $compra = $this->crudService->getFilteredDataSet(self::FILTROCOMPSIMPL, $conditions)->getRows();
        
        if (((is_array($compra) && !is_null($compra)) ? count($compra) : 0) < 1) {
            throw new \Exception("emptyCompraSimplificada");
		}
		
        return $compra[0];
    }
    
    private function stringParaData($data) {
        if(gettype($data) === 'string') {
            $data = substr($data, 0, 10);
            $data = str_replace('/', '-', $data);
            $data = new \DateTime($data);
        }
        
        return $data;
    }
    
    public function getCalculaDataLimiteSeguranca($row)
    {
        if(is_null($row['NRFILTROCOMPSFDA'])) {
            /**
             * 
             * To do 
             * 
             * Buscar o limite de segraça para rotina comum
             * $limi = $this->getLimiSegu($cdfilial, $cdprodini, $cdprodfin, $groupfili);    
             * $date = str_replace("/", "-", $date);
             * 
             **/
        } else {
            $limi = $this->getLimiSeguSimplificado($row['NRFILTROCOMPSFDA']);
            $compra = $this->getCompraSimplificada($row['NRFILTROCOMPSFDA']);
            
            $date = new \DateTime();
        
            $year = $date->format('Y');
            $month = $date->format('m');
            $day = $compra['DIACONSUMOFIN'];
            
            $newDate = DateTime::createFromFormat('Y-n-d', $year.'-'.$month.'-'.$day);
            $newDate = $this->stringParaData($newDate);
        }
        
        $date = $this->somaMes(new \DateTime($newDate), 1);
        
        
        $diaSeguranca = $limi['MAXLIMISEG'] ? $limi['MAXLIMISEG'] : 0;
        $date->add(new \DateInterval('P' . $diaSeguranca . 'D'));
        
        return $date->modify('first day of this month');
         /*
        
        $data = $this->service->getCalculaDataLimiteSeguranca($cdfilial, $cdprodini, $cdprodfin, $cdgroupfili, $dtfin, $nrfiltrocompra);
        $resposta['DATSEGU'] = $data->format('d/m/Y');
        
        */
    }
    
    private function getServicoNaoAprovado()
    {
        $params = array(
    		"P_NRFILTROCOMPSFDA" => $nrfiltro,
    		"NRORG" => $this->environment->getNrorg()
    	);

    	$sql = "SELECT SE.NRORG, SE.CDFILIAL, SE.CDSERVFILI, TO_CHAR(DTMESCALENCARD, 'MM/YYYY') AS DTMESCALENCARD
    	          FROM SERVICO SE, CALENCARD CL, FILGRUPFILI GP, PARAMPLANEJA PP
    	         WHERE (INSTR(:P_CDFILIAL, GP.CDFILIAL)<> 0 or (:P_TCDFILIAL = 'T'))
    	           AND GP.CDFILIAL = CL.CDFILIAL
    	           AND SE.CDFILIAL = CL.CDFILIAL
    	           AND GP.CDFILIAL = PP.CDFILIAL
    	           AND CL.DTMESCALENCARD BETWEEN :P_DTMESINI AND :P_DTMESFIM
    	           AND SE.IDSERVATIV = 'S'
    	           AND SE.IDCARDDESCENTR = 'S'
    	           AND PP.IDBLOQNECCOMP = 'S'
    	           AND (GP.CDGRUPFILI = :P_CDGRUPFILI)
    	           AND EXISTS (SELECT CTRL.CDSERVFILI
                                 FROM CTRLAPROVCARD CTRL
                                WHERE CTRL.CDFILIAL       = SE.CDFILIAL
                                  AND CTRL.CDSERVFILI     = SE.CDSERVFILI
                                  AND CTRL.DTMESCARDAPROV = CL.DTMESCALENCARD
                                  AND CTRL.DTAPROCARDMATR IS NULL)
                 UNION ALL
                SELECT SE.CDFILIAL, SE.CDSERVFILI, TO_CHAR(DTMESCALENCARD, 'MM/YYYY') AS DTMESCALENCARD
                  FROM SERVICO SE, CALENCARD CL, PARAMPLANEJA PP
                 WHERE (INSTR(:P_CDFILIAL, SE.CDFILIAL)<> 0 or (:P_TCDFILIAL = 'T'))
                   AND SE.CDFILIAL = CL.CDFILIAL
                   AND SE.CDFILIAL = PP.CDFILIAL
                   AND CL.DTMESCALENCARD BETWEEN :P_DTMESINI AND :P_DTMESFIM
                   AND SE.IDSERVATIV = 'S'
                   AND SE.IDCARDDESCENTR = 'S'
                   AND PP.IDBLOQNECCOMP = 'S'
                   AND :P_CDGRUPFILI IS NULL
                   AND EXISTS(SELECT CTRL.CDSERVFILI
                                FROM CTRLAPROVCARD CTRL
                               WHERE CTRL.CDFILIAL       = SE.CDFILIAL
                                 AND CTRL.CDSERVFILI     = SE.CDSERVFILI
                                 AND CTRL.DTMESCARDAPROV = CL.DTMESCALENCARD
                                 AND CTRL.DTAPROCARDMATR IS NULL)";
    	$query = $this->connection->prepare($sql);
    	$query->execute($params);
    	return $query->fetchAll();
    }
    
    private function getServicoNaoAprovadoCompraSimplificada() 
    {
        $params = array(
    		"P_NRFILTROCOMPSFDA" => $nrfiltro,
    		"P_DTMESINI" => $nrfiltro,
    		"P_DTMESFIM" => $nrfiltro,
    		"NRORG" => $this->environment->getNrorg()
    	);

    	$sql = "SELECT SE.NRORG, SE.CDFILIAL, SE.CDSERVFILI, TO_CHAR(DTMESCALENCARD, 'MM/YYYY') AS DTMESCALENCARD
                  FROM SERVICO SE JOIN CALENCARD CL     ON SE.CDFILIAL = CL.CDFILIAL
                                  JOIN PARAMPLANEJA PP  ON SE.CDFILIAL = PP.CDFILIAL
                                  JOIN FILICOMPSIMPL FL ON SE.CDFILIAL = FL.CDFILIAL
                 WHERE FL.NRFILTROCOMPSFDA = :P_NRFILTROCOMPSFDA
                   AND CL.DTMESCALENCARD BETWEEN :P_DTMESINI AND :P_DTMESFIM
                   AND SE.IDSERVATIV = 'S'
                   AND SE.IDCARDDESCENTR = 'S'
                   AND PP.IDBLOQNECCOMP = 'S'
                   AND EXISTS (SELECT CTRL.CDSERVFILI
                                 FROM CTRLAPROVCARD CTRL
                                WHERE CTRL.CDFILIAL         = SE.CDFILIAL
                                  AND CTRL.CDSERVFILI       = SE.CDSERVFILI
                                  AND CTRL.DTMESCARDAPROV   = CL.DTMESCALENCARD
                                  AND CTRL.DTAPROCARDMATR IS NULL)";
    	$query = $this->connection->prepare($sql);
    	$query->execute($params);
    	return $query->fetchAll();
    }
    
    public function getCardapioNaoAprovado($row) {
        
        $dtFim = $this->getCalculaDataLimiteSeguranca($row);
        
        /*GetServNAprov(stCdFilial,
                          stTCdFilial,
                          medAgrupFilial.Text,
                          depDataInic.date,
                          SomaMes(depDataFina.Date,1) + StrToIntDef(cdsLimiSegu.FieldByName('MAXLIMISEG').AsString, 0)
                          ));*/    
    }
}