<?php

require_once 'base/GpevinculomEntityBase.php';


/**
 * GpevinculomEntity.
 *
 * @package Entity
 *
 * @author  alessandrobartels
 */
class GpevinculomEntity extends GpevinculomEntityBase
{
    const ORDEM_ALFABETICA = 'ALFABETICA';
    const ORDEM_MATRICULA = 'MATRICULA';
    const ORDEM_DATA_ADMISSAO = 'DATA_ADMISSAO';
    const ORDEM_ESTRUTURA = 'ESTRUTURA';
    const ORDEM_LISTA_VINCULOS = 'LISTA_VINCULOS';


    private $_admitidoPeriodo;
    private $_demitidoPeriodo;
    private $_admitidoMesAnterior;
    private $_demitidoAnteriormente;
    /** @var $_ferias FpaferiasEntity */
    private $_ferias;
    private $movimentacoes;

    /**
     * Enter description here ...
     *
     * @param string $ordem Ordem de pesquisa.
     *
     * @return NULL
     */
    private static function _ordenaQuery($ordem) {
        // FELIPESANTOS 10/02/2015 - Adicionado case de LISTA_VINCULOS

        switch ($ordem) {
            case self::ORDEM_ALFABETICA:
                $nomeColuna = 'GPE_VINCULOM.NMVINCULOM';
                break;
            case self::ORDEM_DATA_ADMISSAO:
                $nomeColuna = 'GPE_VINCULOM.DTADMISSAOVINC';
                break;
            case self::ORDEM_ESTRUTURA:
                $nomeColuna = 'GPE_VINCULOH.NRESTRUTLEGAL';
                break;
            case self::ORDEM_MATRICULA:
                $nomeColuna = 'GPE_VINCULOM.CDMATRICULA';
                break;
            case self::ORDEM_LISTA_VINCULOS:
                $nomeColuna = 'GPE_VINCULOM.NRVINCULOM';
                break;
            default:
                $nomeColuna = null;
        }

        return $nomeColuna;
    }


    /**
     * Retorna array de instancias de vinculos agregados (mestre/historico) conforme parametrizacao.
     *
     * @param integer    $nrorg           Numero da Organizacao.
     * @param TkDateTime $mes_competencia Mes de competencia.
     * @param string     $ds_inicio       Descricao de inicio.
     * @param string     $ds_fim          Descricao final.
     * @param string     $dsOrdem         Ordem de pesquisa.
     *
     * @return array mixed GpevinculomEntity / GpevinculohEntity
     *
     * @author alessandrobartels -
     */
    public function retornaVinculos($nrorg, TkDateTime $mes_competencia, $ds_inicio, $ds_fim, $dsOrdem = null) {
//        $ds_inicio = trim(strtoupper(htmlentities($ds_inicio, null, 'UTF-8')));
//        $ds_fim = trim(strtoupper(htmlentities($ds_fim, null, 'UTF-8')));
        $ds_inicio = trim(strtoupper($ds_inicio));
        $ds_fim = trim(strtoupper($ds_fim));

        $dtUltDiaMes = new TkDateTime($mes_competencia);
        $dtUltDiaMes->setDay(1)->addMonth(1)->subDay(1);

        $condProcesso = 'GPE_VINCULOM.DTADMISSAOVINC <=  \''.$dtUltDiaMes.'\'';

        $condIntervalo = 'UPPER(GPE_VINCULOM.NMVINCULOM) BETWEEN \''.$ds_inicio.'\' AND \''.$ds_fim.'\'';

        $ordenacao = self::_ordenaQuery($dsOrdem);

        if ($ordenacao !== null) {
            $ordenacao = 'ORDER BY '.$ordenacao;
        }

        $resultado = Gpevinculom::retornaVinculos($nrorg, $mes_competencia, $condProcesso, $condIntervalo, $ordenacao);
        return $resultado;
    }




    /**
     * Retorna array de objetos dos vinculos ativos.
     *
     * @param integer    $nrOrg           Numero da Organizacao.
     * @param TkDateTime $mesCompetencia Mes de competencia.
     * @param string     $ds_inicio       Descricao de inicio.
     * @param string     $ds_fim          Descricao final.
     * @param string     $dsOrdem         Parametro de ordenacao.
     *
     * @return array GpevinculomEntity
     *
     * @author alessandrobartels -
     *         alessandrobartels - 21/12/2011
     */
    public function retornaVinculosAtivosMesCompetencia(
        $nrOrg, TkDateTime $mesCompetencia, $ds_inicio, $ds_fim, $dsOrdem, $rescindidos = false, $nrEstrutura = null,
        $verificaConfidencial = true, $paramCaged = false, $nrTipoVinculom = null, $nrVinculoEmpreg = null,
        $nrEstrutSind = null
    ) {
//        $ds_inicio = strtoupper(htmlentities($ds_inicio, null, 'UTF-8'));
//        $ds_fim = strtoupper(htmlentities($ds_fim, null, 'UTF-8'));
        $ds_inicio = trim(strtoupper($ds_inicio));
        $ds_fim = trim(strtoupper($ds_fim));

        $dtUltDiaMes = new TkDateTime($mesCompetencia);
        $dtUltDiaMes->setDay(1)->addMonth(1)->subDay(1);

        // LEONARDOCARVALHO - 22/10/2014 - ADICIONADA CONDIÇÃO DO PARAMCAGED PARA RETORNAR TODOS OS VÍNCULOS COM MOVIMENTAÇÕES NA COMPETÊNCIA
        if ($paramCaged === true) {
            $condProcesso = 'EXISTS (SELECT 1
                                      FROM GPE_MOVIMENTACAO
                                     WHERE GPE_MOVIMENTACAO.NRVINCULOM        = GPE_VINCULOM.NRVINCULOM
                                       AND GPE_MOVIMENTACAO.NRORG             = GPE_VINCULOM.NRORG
                                       AND GPE_MOVIMENTACAO.DTINIMOVIMENT BETWEEN \''.$mesCompetencia.'\' AND \''.$dtUltDiaMes.'\')';
        } else {
            // Condicao anterior onde pega inclusive os vinculos demitidos no mes. Caso seja necessario, colocar uma condicao.
            if ($rescindidos === true) {
                $condProcesso = '(GPE_VINCULOM.DTRESCISAOVINC IS NULL
                                    OR GPE_VINCULOM.DTRESCISAOVINC >= \''.$mesCompetencia.'\'
                                  )
                                  AND GPE_VINCULOM.DTADMISSAOVINC <=  \''.$dtUltDiaMes.'\'';
            } else {
                $condProcesso = '(GPE_VINCULOM.DTRESCISAOVINC IS NULL
                                    OR GPE_VINCULOM.DTRESCISAOVINC > \''.$dtUltDiaMes.'\'
                                  )
                                  AND GPE_VINCULOM.DTADMISSAOVINC <=  \''.$dtUltDiaMes.'\'';
            }
        }

        // BORBA 21/03/2013
        // LEONARDOCARVALHO 01/09/2014 - Alterada a condição do case de ESTRUTURA
        // LEONARDOCARVALHO 12/09/2014 - Adicionado os case de MATRICULA e DATA_ADMISSAO com suas devidas condições
        // FELIPESANTOS 10/02/2015 - Adicionado case de LISTA_VINCULOS com suas devidas condições (Nenhum, todos ou lista de vínculos)
        switch ($dsOrdem){
            case 'ALFABETICA':
                $condIntervalo = 'GPE_VINCULOM.NMVINCULOM BETWEEN \''.$ds_inicio.'\' AND \''.$ds_fim.'\'';
                break;
            case 'ESTRUTURA':
//                $condIntervalo = 'GPE_VINCULOH.NRESTRUTLEGAL BETWEEN '.$ds_inicio.' AND '.$ds_fim.'';
                $condIntervalo  = 'GPE_VINCULOH.NRESTRUTLEGAL = '.$ds_inicio.' ';
                $condIntervalo .= "AND (GPE_VINCULOH.NRESTRUTGEREN = '$ds_fim' OR '$ds_fim' IS NULL)";
                break;
            case 'MATRICULA':
                $condIntervalo = 'GPE_VINCULOM.CDMATRICULA BETWEEN \''.$ds_inicio.'\' AND \''.$ds_fim.'\'';
                break;
            case 'DATA_ADMISSAO':
                $condIntervalo = 'GPE_VINCULOM.DTADMISSAOVINC BETWEEN \''.$ds_inicio.'\' AND \''.$ds_fim.'\'';
                break;
            case 'LISTA_VINCULOS':
                if ($ds_inicio === 'N') {
                    $condIntervalo = "'$ds_inicio' <> 'N'";
                }
                elseif ($ds_inicio === 'A') {
                    $condIntervalo = "'$ds_inicio' = 'A'";
                }
                else {
                    $condIntervalo = "GPE_VINCULOM.NRVINCULOM IN ($ds_inicio)";
                }
                break;
        }

        if ($nrEstrutura !== null){
            $condEstrutura = 'GPE_VINCULOH.NRESTRUTLEGAL = '.$nrEstrutura;
        } else {
            $condEstrutura = null;
        }

        $ordenacao = self::_ordenaQuery($dsOrdem);

        if ($ordenacao !== null) {
            $ordenacao = 'ORDER BY '.$ordenacao;
        }

        //PCK 17/03/2015
        //Inclusão de filtro (em $nrTipoVinculom e $nrVinculoEmpreg) para tipo de vínculo e tipo de vícnulo
        //empregatício. Ambos são passados o número do tipo conforme tabela gpe_vinculom. O valor null significa que o
        //filtro é ignorado.

        $resultado = Gpevinculom::retornaVinculos($nrOrg, $mesCompetencia, $condProcesso, $condIntervalo, $ordenacao, $condEstrutura, $verificaConfidencial, $nrTipoVinculom, $nrVinculoEmpreg, $nrEstrutSind);

        unset($dtUltDiaMes);
        return $resultado;
    }

    public static function getVinculosAtivosParaEvento($nrOrg, TkDateTime $mesCompetencia, $ds_inicio, $ds_fim,
                                                       $nreventom, $tpmovimento, $dsOrdem = null, $dtPagamento = null, $tpvinculo = null )
    {

        $primeiroDiaAno = new TkDateTime($mesCompetencia);
        $primeiroDiaAno->setDay(1)->setMonth(1);

        $dtUltDiaMes = new TkDateTime($mesCompetencia);
        $dtUltDiaMes->setDay(1)->addMonth(1)->subDay(1);
        $ocorrencia     = \FPA::getOcorrenciaCalculo();
        $nrtpmodalidcal = \FPA::getNrModalidadeCalculo();

        /* @todo replicar case de ordenação */
        $condIntervalo = 'AND GPE_VINCULOM.NMVINCULOM BETWEEN \''.$ds_inicio.'\' AND \''.$ds_fim.'\'';
        //ordenar por ordem alfabetica ou estrutura - francis 05/04/2013
        // LEONARDOCARVALHO 16/09/2014
        switch ($dsOrdem){
            case 'ALFABETICA':
                $condIntervalo = 'AND GPE_VINCULOM.NMVINCULOM BETWEEN \''.$ds_inicio.'\' AND \''.$ds_fim.'\'';
                break;
            case 'ESTRUTURA':
//                $condIntervalo = 'GPE_VINCULOH.NRESTRUTLEGAL BETWEEN '.$ds_inicio.' AND '.$ds_fim.'';
                $condIntervalo  = 'AND GPE_VINCULOH.NRESTRUTLEGAL = '.$ds_inicio.' ';
                $condIntervalo .= "AND (GPE_VINCULOH.NRESTRUTGEREN = '$ds_fim' OR '$ds_fim' IS NULL)";
                break;
            case 'MATRICULA':
                $condIntervalo = 'AND GPE_VINCULOM.CDMATRICULA BETWEEN \''.$ds_inicio.'\' AND \''.$ds_fim.'\'';
                break;
            case 'DATA_ADMISSAO':
                $condIntervalo = 'AND GPE_VINCULOM.DTADMISSAOVINC BETWEEN \''.$ds_inicio.'\' AND \''.$ds_fim.'\'';
                break;
        }

        //gabrielcapanema - 21/05/2015 : filtra por tipo vínculo CNAB
        if($tpvinculo){
            $condIntervalo .= 'AND GPE_VINCULOM.NRTIPOVINCULOM IN ('.$tpvinculo.')';
        }

        $ordenacao = 'ORDER BY GPE_VINCULOM.NMVINCULOM';

        // felipesantos - 08/04/2015: incluída data de pagamento
        $sql = <<<SQL

SELECT MAXVINCH.DTMESCOMPETENC,
  GPE_VINCULOM.*
FROM GPE_VINCULOM
INNER JOIN
  (SELECT MAX(DTMESCOMPETENC) DTMESCOMPETENC,
    NRVINCULOM,
    NRORG
  FROM GPE_VINCULOH
    WHERE DTMESCOMPETENC   >= '$primeiroDiaAno'
      AND DTMESCOMPETENC   <= '$dtUltDiaMes'
      AND NRORG             = $nrOrg
  GROUP BY NRVINCULOM,
    NRORG
  ) MAXVINCH
ON (MAXVINCH.NRVINCULOM = GPE_VINCULOM.NRVINCULOM
AND MAXVINCH.NRORG      = GPE_VINCULOM.NRORG)
INNER JOIN GPE_VINCULOH
ON GPE_VINCULOH.NRVINCULOM      = MAXVINCH.NRVINCULOM
AND GPE_VINCULOH.DTMESCOMPETENC = MAXVINCH.DTMESCOMPETENC
AND GPE_VINCULOH.NRORG          = MAXVINCH.NRORG
AND GPE_VINCULOH.CONFIDENCIAL   = 'N'
JOIN FPA_ITECALCFOLHA
ON GPE_VINCULOM.NRORG       =FPA_ITECALCFOLHA.NRORG
AND GPE_VINCULOM.NRVINCULOM = FPA_ITECALCFOLHA.NRVINCULOM
JOIN FPA_CALCULOFOLHA
ON FPA_CALCULOFOLHA.NRORG           = FPA_ITECALCFOLHA.NRORG
AND FPA_CALCULOFOLHA.NRCALCULOFOLHA = FPA_ITECALCFOLHA.NRCALCULOFOLHA
WHERE GPE_VINCULOM.NRORG = $nrOrg
AND FPA_ITECALCFOLHA.NREVENTOM      = '$nreventom'
AND FPA_ITECALCFOLHA.VREVENTOFOLHA  <> 0
AND FPA_ITECALCFOLHA.DTOCORRENCIA = NVL('$dtPagamento', FPA_ITECALCFOLHA.DTOCORRENCIA)
AND FPA_CALCULOFOLHA.NRTIPOMOVIMENT = '$tpmovimento'
AND FPA_CALCULOFOLHA.NRTPMODALIDCAL = '$nrtpmodalidcal'
AND FPA_CALCULOFOLHA.DTOCORRENCIA BETWEEN '$mesCompetencia' AND '$dtUltDiaMes'
AND FPA_CALCULOFOLHA.nrocorrecal = '$ocorrencia'
SQL;

        $sql = $sql.' '.$condIntervalo.' '.$ordenacao;

        $result = Gpevinculom::_retornaVinculos($nrOrg, $sql, $mesCompetencia);
        return $result;
    }

    /**
     * Retorna array de objetos dos vinculos demitidos.
     *
     * @param integer    $nrOrg           Numero da organizacao.
     * @param TkDateTime $mesCompetencia Mes de competencia.
     * @param string     $ds_inicio       Descricao de inicio.
     * @param string     $ds_fim          Descricao final.
     * @param TkDateTime $dt_ini          Data de inicio do periodo.
     * @param TkDateTime $dt_fim          Data fim do periodo.
     * @param string     $dsOrdem         Ordem de pesquisa.
     *
     * @return array GpevinculomEntity
     *
     * @author alessandrobartels -
     *         alessandrobartels - 21/12/2011
     */
    public function retornaVinculosDemitidosMesCompetencia(
        $nrOrg, TkDateTime $mesCompetencia, $ds_inicio, $ds_fim, TkDateTime $dt_ini, TkDateTime $dt_fim, $dsOrdem,
        $nrEstrutura = null
    ) {

//        $ds_inicio = strtoupper(htmlentities($ds_inicio, null, 'UTF-8'));
//        $ds_fim = strtoupper(htmlentities($ds_fim, null, 'UTF-8'));
        $ds_inicio = trim(strtoupper($ds_inicio));
        $ds_fim = trim(strtoupper($ds_fim));

        // BORBA 05/08/2013 Pega ultimo dia do mes de competência.
        $ultimoDiaMesCompetencia = clone $mesCompetencia;
        $ultimoDiaMesCompetencia->setDay(1)->addMonth(1)->subDay(1);

        if (($dt_ini === NULL) || ($dt_fim === NULL)) {
            $condProcesso = '(GPE_VINCULOM.DTRESCISAOVINC IS NOT NULL
                  		  AND (GPE_VINCULOM.DTRESCISAOVINC BETWEEN \''.$mesCompetencia.'\' AND \''.$ultimoDiaMesCompetencia.'\'))
                         AND GPE_VINCULOM.DTADMISSAOVINC <=  \''.$ultimoDiaMesCompetencia.'\'';
        }else{
            $condProcesso = '(GPE_VINCULOM.DTRESCISAOVINC IS NOT NULL
                              AND (GPE_VINCULOM.DTRESCISAOVINC BETWEEN \''.$dt_ini.'\' AND \''.$dt_fim.'\'))
                             AND GPE_VINCULOM.DTADMISSAOVINC <=  \''.$ultimoDiaMesCompetencia.'\'';
        }


        /* @todo replicar case de ordenação */
        $condIntervalo = 'GPE_VINCULOM.NMVINCULOM BETWEEN \''.$ds_inicio.'\' AND \''.$ds_fim.'\'';
        //ordenar por ordem alfabetica ou estrutura - francis 05/04/2013
        // LEONARDOCARVALHO 16/09/2014
        switch ($dsOrdem){
            case 'ALFABETICA':
                $condIntervalo = 'GPE_VINCULOM.NMVINCULOM BETWEEN \''.$ds_inicio.'\' AND \''.$ds_fim.'\'';
                break;
            case 'ESTRUTURA':
//                $condIntervalo = 'GPE_VINCULOH.NRESTRUTLEGAL BETWEEN '.$ds_inicio.' AND '.$ds_fim.'';
                $condIntervalo  = 'GPE_VINCULOH.NRESTRUTLEGAL = '.$ds_inicio.' ';
                $condIntervalo .= "AND (GPE_VINCULOH.NRESTRUTGEREN = '$ds_fim' OR '$ds_fim' IS NULL)";
                break;
            case 'MATRICULA':
                $condIntervalo = 'GPE_VINCULOM.CDMATRICULA BETWEEN \''.$ds_inicio.'\' AND \''.$ds_fim.'\'';
                break;
            case 'DATA_ADMISSAO':
                $condIntervalo = 'GPE_VINCULOM.DTADMISSAOVINC BETWEEN \''.$ds_inicio.'\' AND \''.$ds_fim.'\'';
                break;
        }

        if ($nrEstrutura !== null){
            $condEstrutura = 'GPE_VINCULOH.NRESTRUTLEGAL = '.$nrEstrutura;
        } else {
            $condEstrutura = null;
        }

        $ordenacao = self::_ordenaQuery($dsOrdem);

        if ($ordenacao !== null) {
            $ordenacao = 'ORDER BY '.$ordenacao;
        }
		
        $resultado = Gpevinculom::retornaVinculos($nrOrg, $mesCompetencia, $condProcesso, $condIntervalo, $ordenacao, $condEstrutura);

        unset($dtUltDiaMes);

        return $resultado;
    }
	
    /**
     * Retorna array de objetos dos vinculos demitidos (todos os vinculos demitidos independente da competencia).
     *
     * @param integer    $nrOrg           Numero da organizacao.
     * @param TkDateTime $mesCompetencia  Mes de competencia.
     * @param string     $ds_inicio       Descricao de inicio.
     * @param string     $ds_fim          Descricao final.
     * @param TkDateTime $dt_ini          Data de inicio do periodo.
     * @param TkDateTime $dt_fim          Data fim do periodo.
     * @param string     $dsOrdem         Ordem de pesquisa.
     *
     * @return array GpevinculomEntity
     *
     * @author marcelosilva -
     *         marcelosilva - 20/11/2014
     */
    public function retornaVinculosDemitidosGeral(
        $nrOrg, TkDateTime $mesCompetencia, $ds_inicio, $ds_fim, TkDateTime $dt_ini, TkDateTime $dt_fim, $dsOrdem,
        $nrEstrutura = null
    ) {

//      $ds_inicio = strtoupper(htmlentities($ds_inicio, null, 'UTF-8'));
//      $ds_fim = strtoupper(htmlentities($ds_fim, null, 'UTF-8'));
        $ds_inicio = trim(strtoupper($ds_inicio));
        $ds_fim = trim(strtoupper($ds_fim));

        // BORBA 05/08/2013 Pega ultimo dia do mes de competência.
        $ultimoDiaMesCompetencia = clone $mesCompetencia;
        $ultimoDiaMesCompetencia->setDay(1)->addMonth(1)->subDay(1);

        if (($dt_ini === NULL) || ($dt_fim === NULL)) {
            $condProcesso = '(GPE_VINCULOM.DTRESCISAOVINC IS NOT NULL
                          AND GPE_VINCULOM.DTADMISSAOVINC <=  \''.$ultimoDiaMesCompetencia.'\')';
        }else{
            $condProcesso = '(GPE_VINCULOM.DTRESCISAOVINC IS NOT NULL
                          AND GPE_VINCULOM.DTADMISSAOVINC <=  \''.$ultimoDiaMesCompetencia.'\')';
        }


        /* @todo replicar case de ordenação */
        $condIntervalo = 'GPE_VINCULOM.NMVINCULOM BETWEEN \''.$ds_inicio.'\' AND \''.$ds_fim.'\'';
        //ordenar por ordem alfabetica ou estrutura - francis 05/04/2013
        // LEONARDOCARVALHO 16/09/2014
        switch ($dsOrdem){
            case 'ALFABETICA':
                $condIntervalo = 'GPE_VINCULOM.NMVINCULOM BETWEEN \''.$ds_inicio.'\' AND \''.$ds_fim.'\'';
                break;
            case 'ESTRUTURA':
//                $condIntervalo = 'GPE_VINCULOH.NRESTRUTLEGAL BETWEEN '.$ds_inicio.' AND '.$ds_fim.'';
                $condIntervalo  = 'GPE_VINCULOH.NRESTRUTLEGAL = '.$ds_inicio.' ';
                $condIntervalo .= "AND (GPE_VINCULOH.NRESTRUTGEREN = '$ds_fim' OR '$ds_fim' IS NULL)";
                break;
            case 'MATRICULA':
                $condIntervalo = 'GPE_VINCULOM.CDMATRICULA BETWEEN \''.$ds_inicio.'\' AND \''.$ds_fim.'\'';
                break;
            case 'DATA_ADMISSAO':
                $condIntervalo = 'GPE_VINCULOM.DTADMISSAOVINC BETWEEN \''.$ds_inicio.'\' AND \''.$ds_fim.'\'';
                break;
        }

        if ($nrEstrutura !== null){
            $condEstrutura = 'GPE_VINCULOH.NRESTRUTLEGAL = '.$nrEstrutura;
        } else {
            $condEstrutura = null;
        }

        $ordenacao = self::_ordenaQuery($dsOrdem);

        if ($ordenacao !== null) {
            $ordenacao = 'ORDER BY '.$ordenacao;
        }
        //@todo descobrir quem fez pra saber o porque. ??? não está sendo usada dentro do retorna vínculos, está sendo passado na mesma posição do confidencial O_o*
        $desconsideraCompetencia = true;
        $resultado = Gpevinculom::retornaVinculos($nrOrg, $mesCompetencia, $condProcesso, $condIntervalo, $ordenacao, $condEstrutura, $desconsideraCompetencia);

        unset($dtUltDiaMes);

        return $resultado;
    }

    /**
     * Retorna array de objetos dos vinculos demitidos e admitidos no mes da competencia.
     *
     * @param integer    $nrOrg           Numero da organizacao.
     * @param TkDateTime $mesCompetencia Mes de competencia.
     * @param string     $ds_inicio       Descricao de inicio.
     * @param string     $ds_fim          Descricao final.
     * @param TkDateTime $dt_ini          Data de inicio do periodo.
     * @param TkDateTime $dt_fim          Data fim do periodo.
     * @param string     $dsOrdem         Ordem de pesquisa.
     *
     * @return array GpevinculomEntity
     *
     * @author leonardocarvalho 14/08/2014
     */
    public function retornaVinculosDemitidosAdmitidosMesCompetencia(
        $nrOrg, TkDateTime $mesCompetencia, $ds_inicio, $ds_fim, TkDateTime $dt_ini, TkDateTime $dt_fim, $dsOrdem,
        $nrEstrutura = null, $paramCaged = null
    ) {

        $ds_inicio = trim(strtoupper($ds_inicio));
        $ds_fim = trim(strtoupper($ds_fim));

        $ultimoDiaMesCompetencia = clone $mesCompetencia;
        $ultimoDiaMesCompetencia->setDay(1)->addMonth(1)->subDay(1);

        if (($dt_ini === NULL) || ($dt_fim === NULL)) {
            $condProcesso = '(GPE_VINCULOM.DTRESCISAOVINC IS NOT NULL
                  		  AND (GPE_VINCULOM.DTRESCISAOVINC BETWEEN \''.$mesCompetencia.'\' AND \''.$ultimoDiaMesCompetencia.'\'))
                         OR GPE_VINCULOM.DTADMISSAOVINC BETWEEN \''.$mesCompetencia.'\' AND \''.$ultimoDiaMesCompetencia.'\'';
        }else{
            $condProcesso = '(GPE_VINCULOM.DTRESCISAOVINC IS NOT NULL
                              AND (GPE_VINCULOM.DTRESCISAOVINC BETWEEN \''.$dt_ini.'\' AND \''.$dt_fim.'\'))
                             OR GPE_VINCULOM.DTADMISSAOVINC BETWEEN \''.$mesCompetencia.'\' AND \''.$ultimoDiaMesCompetencia.'\'';
        }

        $condIntervalo = '';

        // LEONARDOCARVALHO 16/09/2014
        switch ($dsOrdem){
            case 'ALFABETICA':
                $condIntervalo = 'GPE_VINCULOM.NMVINCULOM BETWEEN \''.$ds_inicio.'\' AND \''.$ds_fim.'\'';
                break;
            case 'ESTRUTURA':
//                $condIntervalo = 'GPE_VINCULOH.NRESTRUTLEGAL BETWEEN '.$ds_inicio.' AND '.$ds_fim.'';
                $condIntervalo  = 'GPE_VINCULOH.NRESTRUTLEGAL = '.$ds_inicio.' ';
                $condIntervalo .= "AND (GPE_VINCULOH.NRESTRUTGEREN = '$ds_fim' OR '$ds_fim' IS NULL)";
                break;
            case 'MATRICULA':
                $condIntervalo = 'GPE_VINCULOM.CDMATRICULA BETWEEN \''.$ds_inicio.'\' AND \''.$ds_fim.'\'';
                break;
            case 'DATA_ADMISSAO':
                //LEONARDOCARVALHO 01/10/2014 - Necessário anular o $condProcesso pois essa opção deve retornar apenas vínculos no período informado
                $condProcesso = null;
                $condIntervalo = 'GPE_VINCULOM.DTADMISSAOVINC BETWEEN \''.$ds_inicio.'\' AND \''.$ds_fim.'\'';
                break;
        }

        // LEONARDOCARVALHO 01/10/2014 - Criado parâmetro e condição para filtrar vínculos com ou sem seguro desemprego
        if ($paramCaged !== null){
            $condIntervalo .= ' AND GPE_VINCULOM.IDSEGDESEMPVINC = \''.$paramCaged.'\'';
        }

        if ($nrEstrutura !== null){
            $condEstrutura = 'GPE_VINCULOH.NRESTRUTLEGAL = '.$nrEstrutura;
        } else {
            $condEstrutura = null;
        }

        $ordenacao = self::_ordenaQuery($dsOrdem);

        if ($ordenacao !== null) {
            $ordenacao = 'ORDER BY '.$ordenacao;
        }

        $resultado = Gpevinculom::retornaVinculos($nrOrg, $mesCompetencia, $condProcesso, $condIntervalo, $ordenacao, $condEstrutura);

        unset($dtUltDiaMes);

        return $resultado;
    }

    /**
     * Ajusta intervalo das datas de admissao e recisao.
     *
     * @param TkDateTime &$data_ini Data de inicio do periodo.
     * @param TkDateTime &$data_fim Data final do periodo.
     *
     * @return void
     *
     * @author alessandrobartels -
     */
    public function ajustaIntervaloAdmResc(TkDateTime &$data_ini, TkDateTime &$data_fim, $controleAdmissao) {
        $data_admissao = new TkDateTime($this->getDtadmissaovinc(), TkDateTime::FORMAT_DATE);
        $data_rescisao = null;

        if ($this->getDtrescisaovinc() !== false) {
            $data_rescisao = new TkDateTime($this->getDtrescisaovinc(), TkDateTime::FORMAT_DATE);
        }

        $this->_admitidoPeriodo = false;
        $this->_demitidoPeriodo = false;
        $this->_admitidoMesAnterior = false;
        $this->_demitidoAnteriormente = false;
        if ($data_admissao->isLater($data_ini) === true) {
            if (($data_admissao->isEarlier($data_fim) === true)
                ||($data_admissao->equals($data_fim) ===true)
            ) {
                $data_ini = $data_admissao; // Verificar.
                $this->_admitidoPeriodo = true;
                /* alessandrobartels - 03/09/2012
                 * O controle de adimissão determina se calcula o mes com 30 ou 31 dias no caso de admissao.
                 */
                //BORBA 25/07/2014 Colocado o if para limitar o controle admissão para que este nao seja maior o ultimo dia do mes.
                if($controleAdmissao<=$data_fim->getDay()){
                    $data_fim->setDay($controleAdmissao);
                }
            }
        } else {
            // Decrementa 1 mes da data.
            $data_ini_mes_anterior = new TkDateTime($data_ini, TkDateTime::FORMAT_DATE);
            $data_ini_mes_anterior->subMonth(1)->setDay(1);
            if (($data_admissao->isLater($data_ini_mes_anterior) === true)
                ||($data_admissao->equals($data_ini_mes_anterior) === true)
            ) {
                $this->_admitidoMesAnterior = true;
            }
        }

        if ($data_rescisao !== null) {
            if ($data_rescisao->isEarlier($data_fim) === true) {
                if (($data_rescisao->isLater($data_ini) === true)
                    ||($data_rescisao->equals($data_ini) === true)
                ) {
                    $data_fim->set($data_rescisao);
                    /* @todo Analisar calendario para projetar desligamento no caso de fim de semana e feriado.*/
                    $this->_demitidoPeriodo = true;
                } else {
                    $this->_demitidoAnteriormente = true;
                    //BORBA 19/05/2015 Mesmo não satisfazendo a condição seta a data fim como sendo a rescisão porém ressalva como demitido anteriormente.
                    $data_fim->set($data_rescisao);
                }
            }
        }
    }


    /**
     * Retorna o periodo em dias da duracao do vinculo.
     *
     * @param string $dtAdmissaoVinc Data de Admissao do Vinculo.
     * @param string $dtRescisaoVinc Data de Recisao do Vinculo.
     *
     * @return number
     *
     * @author brunofeliciano - 14/07/2011
     */
    public function retornaDuracaoVinculo($dtAdmissaoVinc, $dtRescisaoVinc){
        $aux = new TkDateTime($dtRescisaoVinc, TkDateTime::FORMAT_DATE);
        $aux2 = new TkDateTime($dtAdmissaoVinc, TkDateTime::FORMAT_DATE);
        $periodo = $aux2->daysAfter($aux);
        return $periodo;
    }


    /**
     * Enter description here ...
     *
     * @param integer $nrpessoa   Nr Pessoa.
     * @param integer $nrorgTrab  Nr Org.
     * @param string  $dtAdmissao Data de admissao do vinculo.
     * @param string  $dtRescisao Data de rescisao do vinculo.
     * @param string  $matricula  Matricula.
     *
     * @return boolean
     *
     * @author luistavares 12/08/2011
     *         brunofeliciano - 30/08/2011 - Atribuido data de admissao do vinculo (Dtadmissaovinc).
     *         alessandrobartels - 23/09/2011 - Retirado o numero do vinculoh e dtinclusao por estar usando strategy.
     */
    public function novoMestre(
        $nrpessoa, $nrorgTrab, $dtAdmissao, $dtRescisao, $matricula = null, $tipoVinculo = null,
        $nrvinculoempreg = null, $nrtpadmissao = null, $nmvinculom = null
    ){
        try{
            $vinculo = new self();

            $vinculo->setNrpessoa($nrpessoa);
            $vinculo->setNrorg($nrorgTrab);
            $vinculo->setDtadmissaovinc($dtAdmissao);
            $vinculo->setDtrescisaovinc($dtRescisao);
            $vinculo->setCdmatricula($matricula);
            $vinculo->setNrtipovinculom($tipoVinculo);
            $vinculo->setNrvinculoempreg($nrvinculoempreg);
            $vinculo->setNrtpadmissao($nrtpadmissao);
            $vinculo->setNmvinculom($nmvinculom);



            $vinculo->persist();
            // Verificar o retorno do numero do vinculo.
            $nrVinculo = $vinculo->getNrvinculom();
            unset($vinculo);
            return $nrVinculo;
        }
        catch (Exception $e){
            $e = new Exception('Erro ao criar um novo vinculo. Nao pode inserir um novo mestre.');
            throw $e;
        }

    }


    /**
     * Enter description here ...
     *
     * @param integer $nrvinculom Nr Vinculo.
     * @param integer $nrorgTrab  Nr Org.
     *
     * @return boolean
     *
     * @author luistavares 12/08/2011
     */
    public function destroiMestre($nrvinculom, $nrorgTrab){
        try{
            $vinculom = Gpevinculom::retornaVinculo($nrvinculom, $nrorgTrab);
            if (is_object($vinculom) === true) {
                /* Adiciona um item a lista, apos isso, o item eh removido da lista
                 * e enviado para lista de itens removidos, lista essa que ira apagar
                 * todos seus itens quando executar o persist(). */
                $this->addItem($vinculom, $nrvinculom);
                $this->removeItem($vinculom);
                $this->persist();

                return true;
            } else {
                return false;
            }
        }
        catch (Exception $e){
            throw new Exception('Nao foi possivel excluir o vinculo.');
        }

    }


    /**
     * Retorna vinculo pela chave.
     *
     * @param integer $nrvinculom Numero do Vinculo.
     * @param integer $nrorg      Numero da Organizacao.
     *
     * @return GpevinculomEntity
     *
     * @author brunofeliciano - 30/08/2011
     */
    public function retornaVinculo($nrvinculom, $nrorg){
        return Gpevinculom::retornaVinculo($nrvinculom, $nrorg);
    }


    /**
     * Retorna a instancia de Vinculo por pessoa.
     *
     * @param integer $nrOrg     Numero de Organizacao.
     * @param string  $matricula Matricula.
     * @param string  $estrutura Estrutura.
     *
     * @return Gpevinculom
     *
     * @author talitaorfano 06/09/2011
     *         thalesvalias 26/11/2012 - Estrutura passa a ser considerada.
     */
    public function retornaVinculoPorMatriculaEstrutura($nrOrg, $matricula, $estrutura){
        $vinculo = Gpevinculom::retornaVinculoPorMatriculaEstrutura($nrOrg, $matricula, $estrutura);
        $vinculo = end($vinculo);
        return $vinculo;
    }


    /**
     * Enter description here ...
     *
     * @param integer $nrOrg    Numero da Organizacao.
     * @param integer $nrPessoa Numero da Pessoa.
     *
     * @return Ambigous <multitype:, NULL, TkModelEntity>
     */
    public function retornaVinculosPorPessoa($nrOrg, $nrPessoa){
        return Gpevinculom::retornaVinculosPorPessoa($nrOrg, $nrPessoa);
    }


    /**
     * Enter description here ...
     *
     * @param integer $nrOrg Numero da Organizacao.
     *
     * @return mixed
     */
    public function retornaVinculoMestre($nrOrg){
        return end(Gpevinculom::retornaVinculoMestre($nrOrg));
    }


    /**
     * Enter description here ...
     *
     * @param integer $nrOrg Numero da Organizacao.
     *
     * @return integer
     */
    public function retornaPK($nrOrg){
        $vinculo = $this->retornaVinculoMestre($nrOrg);
        return $vinculo->getNrvinculom();
    }

    /**
     * @param $nrorg
     * @param $matricula
     * @return GpevinculomEntity
     */
    public static function getVinculoMatricula($nrorg, $matricula) {
        $gpevinculom = self::getEntityByWhere('NRORG = ? AND CDMATRICULA = ?', array($nrorg, $matricula));
        return $gpevinculom;
    }

    public function insereVinculo($nrorg, $nrpessoa, $dtadmissao, $dtrescisao, $matricula, $tipovinculo,
                                  $nrvinculoempreg, $nrtpadmissao, $nmvinculom, $dtavisoprevio, $nrtpdemissao,
                                  $dtexamemedico)
    {
        $this->setNrorg($nrorg);
        $this->setNrpessoa($nrpessoa);
        $this->setNrtipovinculom($tipovinculo);
        $this->setNrvinculoempreg($nrvinculoempreg);
        $this->setNrtpadmissao($nrtpadmissao);
        $this->setNrtpdemissao($nrtpdemissao);
        $this->setNmvinculom($nmvinculom);
        $this->setCdmatricula($matricula);
        $this->setDtadmissaovinc($dtadmissao);
        $this->setDtrescisaovinc($dtrescisao);
        $this->setDtavisoprevio($dtavisoprevio);
        $this->setDtexamemedico($dtexamemedico);
    }

    /**
     * @param $dtmescompetencia
     * @return GpevinculohEntity
     */
    public function getUltimoHistoricoCompetencia($dtmescompetencia) {
        $historico = Gpevinculoh::getUltimoHistoricoCompetencia(
            $this->getNrorg(), $this->getNrvinculom(), $dtmescompetencia
        );
        return $historico;
    }

    public function getFeriasNaoGozadas() {
        if (!isset($this->_ferias)) {
            $arrayFerias = Fpaferias::retornaItem($this->getNrvinculom(), $this->getNrorg());
            if ($arrayFerias) {
                $this->_ferias = new FpaferiasEntity();
            }
            foreach($arrayFerias as $ferias) {
                if (($ferias->getIdcontrolferias() !== FpaferiasEntity::IDCONTROLFERIAS_GOZADO_INTEGRAL) &&
                    ($ferias->getIdcontrolferias() !== FpaferiasEntity::IDCONTROLFERIAS_PERDIDO))
                {
                    $this->_ferias->addItem($ferias, $ferias->getNrferias());
                }
            }
        }
        return $this->_ferias;
    }


    public function getMovimentacoes($dtcompetencia) {
        $movimentacoes = Gpemovimentacao::retornaMovimentacoes(
            $this->getNrorg(), $this->getNrvinculom(), $dtcompetencia
        );
        return $movimentacoes;
    }



    /*Gabriel Capanema 13/04/2015
    getEscala, getOcupacao
    para tela de conferencia de migração */
    public function getTipovinculo() {
        $tipovinculo = $this->getNrtipovinculom();
        return $tipovinculo;
    }

    public function getEscala() {
        $escala = GpeAlteescala::retornaAlteracaoEscalaPorVinculo(
            $this->getNrorg(), $this->getNrvinculom()
        );
        return $escala;
    }

    public function getOcupacao() {
        $ocupacao = GpeAlteocupacao::retornaAlteracaoOcupacaoVinculo(
            $this->getNrorg(), $this->getNrvinculom()
        );
        return $ocupacao;
    }

    /**
     * @return \GpealtesalarioEntity[]
     */
    public function getAlteracoesSalario() {
        $alteSalario = \Gpealtesalario::retornaAlteSalario($this->getNrorg(), $this->getNrvinculom());
        return $alteSalario;
    }

    /**
     * @return \GpealtesitufuncEntity[]
     */
    public function getSituacoesFuncionais() {
        $alteSalario = \Gpealtesitufunc::getSituacoesVinculo($this->getNrorg(), $this->getNrvinculom());
        return $alteSalario;
    }

    /**
     * @param $dtcompetencia
     * @return \GpealtesitufuncEntity
     */
    public function getUltimaSituacaoFuncional($dtcompetencia) {
        $altesitufunc = \Gpealtesitufunc::getUltimaSituacaoPorDatainicio(
            $this->getNrorg(), $this->getNrvinculom(), $dtcompetencia
        );
        return $altesitufunc;
    }

}